{+--------------------------------------------------------------------------+
 | Component:   TmwCustomFormComponent
 | Author:      Martin Waldenburg
 | Copyright    1997, all rights reserved.
 | Description: Base class for forms, which are components.
 |              The FormComponents will have a restricted visual representation
 |              as stored in the FormComponent's .DFM file, if dropped on a form.
 | Version:     1.5
 | Status:      Freeware
 | DISCLAIMER: This is provided as is, expressly without a warranty of any kind.
 |             You use it at your own risc.
 +--------------------------------------------------------------------------+}


unit mwCuFoCo;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms,  ComCtrls, Menus;

type
  TmwCustomFormComponent = class(TForm)
  private
    FRepresentation: Boolean;
    Dummy: TForm;
    DummyRect: TRect;
    ContRect: TRect;
    fLeftBorder: Integer;
    fTopBorder: Integer;
    function RunTime: Boolean;
    procedure CreateDummyForm;
    procedure DummyResizeHandler(Sender: TObject);
    procedure SetDummyBounds;
    procedure SetControlBounds;
    procedure SetRepresentation(newValue: Boolean);
    procedure DummyClickHandler(Sender: TObject);
  protected
    procedure SetBounds(Left, Top, Width, Height: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateWindowHandle(const Params: TCreateParams); override;
  published
    property Align;
    property ControlStyle;
    property IsControl;
    property MouseCapture;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property DisplayRepresentation: Boolean read FRepresentation write SetRepresentation;
  end;

var
  mwCustomFormComponent: TmwCustomFormComponent;

implementation

{$R *.DFM}

constructor TmwCustomFormComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption];
  Caption:='';
  fLeftBorder:= GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXFIXEDFRAME);
  fTopBorder:= GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CXBORDER) +
               GetSystemMetrics(SM_CXFIXEDFRAME); { use SM_CXFRAME for Windows 3.1 }
  if not RunTime then
    begin
      Height:= 100;
      Width:= 200;
    end;
end;  { Create }

destructor TmwCustomFormComponent.Destroy;
begin
  if not RunTime then
    begin
      ControlStyle := ControlStyle - [csFramed];
      (Owner as TForm).Repaint;
    end;
  Dummy.Free;
  Parent:= nil;
  inherited Destroy;
end;  { Destroy }

procedure TmwCustomFormComponent.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited CreateWindowHandle(Params);
  if not RunTime then
    begin
      Dummy.OnClick := DummyClickHandler;
      Dummy.OnResize := DummyResizeHandler;
    end;
end;  { CreateWindowHandle }

function TmwCustomFormComponent.RunTime: Boolean;
begin
  RunTime := not (csDesigning in ComponentState);
end;  { RunTime }

procedure TmwCustomFormComponent.SetBounds(Left, Top, Width, Height: Integer);
begin
  inherited SetBounds(Left, Top, Width, Height);
  if not RunTime then
    begin
      if Dummy = nil then  CreateDummyForm;
      SetDummyBounds;
      (Owner as TForm).Repaint;
    end
  else
    begin
      (Owner as TForm).Repaint;
    end;
end;  { SetBounds }

procedure TmwCustomFormComponent.CreateDummyForm;
begin
  if not RunTime then
    begin
      if Dummy <> nil then  Dummy.Free;
      Dummy:= TForm.Create(mwCustomFormComponent);
      ReadComponentRes(Self.ClassName, Dummy);
      Dummy.ControlStyle := Dummy.ControlStyle - [csAcceptsControls];
      Dummy.BorderIcons:= Dummy.BorderIcons - Dummy.BorderIcons + [biSystemMenu];
      Dummy.FormStyle:= fsStayOnTop;
      Dummy.Caption:= Dummy.Caption + ' -Representation';
    end;
end;  { CreateDummyForm }

procedure TmwCustomFormComponent.SetRepresentation(newValue: Boolean);
begin
  if not RunTime then
    begin
      fRepresentation:= newValue;
      if Dummy = nil then  CreateDummyForm;
      Dummy.Color:= Color;
      Dummy.BorderStyle:= bsSizeable; { Don't set any other value !!! }
      if fRepresentation then Dummy.Show else Dummy.Hide;
    end;
end;  { GetRepresentation }

procedure TmwCustomFormComponent.DummyClickHandler(Sender: TObject);
begin
  if (not RunTime) then
    begin
      SetControlBounds;
    end;
end;  { DummyClickHandler }

procedure TmwCustomFormComponent.DummyResizeHandler(Sender: TObject);
begin
  if (not RunTime) then
    begin
      SetControlBounds;
    end;
end;  { DummyResizeHandler }


procedure TmwCustomFormComponent.SetDummyBounds;
var
  ScrPos: TPoint;
  CliPos: TPoint;
begin
  if not RunTime then
    begin
      ContRect:= BoundsRect;
      DummyRect.Left:= ContRect.Left + (Owner as TForm).Left + fLeftBorder;
      DummyRect.Top:= ContRect.Top + (Owner as TForm).Top + fTopBorder;
      DummyRect.Right:= ContRect.Right + (Owner as TForm).Left + fLeftBorder;
      DummyRect.Bottom:= ContRect.Bottom + (Owner as TForm).Top + fTopBorder;
      Dummy.BoundsRect:= DummyRect;
    end;
end;  { SetDummyBounds }

procedure TmwCustomFormComponent.SetControlBounds;
begin
  if not RunTime then
    begin
      DummyRect:= Dummy.BoundsRect;
      ContRect.Left:= DummyRect.Left - (Owner as TForm).Left - fLeftBorder;
      ContRect.Top:= DummyRect.Top - (Owner as TForm).Top - fTopBorder;
      ContRect.Right:= DummyRect.Right - (Owner as TForm).Left - fLeftBorder;
      ContRect.Bottom:= DummyRect.Bottom - (Owner as TForm).Top - fTopBorder;
      BoundsRect:= ContRect;
    end;
end;  { SetControlBounds }

end.
{ Martin Waldenburg
  Landaeckerstrasse 27
  71642 Ludwigsburg
  Germany  }
